/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.se.eip.listener;

import static org.ow2.petals.se.eip.EIPConstants.ACTION;

import java.util.List;
import java.util.logging.Level;

import javax.jbi.messaging.MessagingException;
import javax.jbi.servicedesc.ServiceEndpoint;

import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.jbidescriptor.generated.Consumes;
import org.ow2.petals.component.framework.listener.AbstractJBIListener;
import org.ow2.petals.component.framework.su.ServiceUnitDataHandler;
import org.ow2.petals.component.framework.su.ServiceUnitManager;
import org.ow2.petals.se.eip.EIPComponent;
import org.ow2.petals.se.eip.ExchangeContext;
import org.ow2.petals.se.eip.patterns.Pattern;

/**
 *
 * @author alouis
 *
 */
public class JBIListener extends AbstractJBIListener implements ExchangeContext {

	@Override
	protected void onJBIMessage(Exchange exchange) {
	    
		if (exchange.isProviderRole()) {
			process(exchange);
		} else {
			processAsyncResponse(exchange);
		}
	}

	/**
	 * Check for the action defined in the SU and call the related Pattern
	 * processor
	 *
	 * @param exchange
	 */
	protected void process(Exchange exchange) {
		String action = getExtensions().get(ACTION);
		Pattern pattern = ((EIPComponent) getComponent()).getPatterns().get(
				action.toLowerCase());
		if (pattern != null) {
			pattern.processPattern(exchange, this);
		} else {
			try {
				String error = "action '" + action
						+ "' defined for the endpoint '"
						+ exchange.getEndpointName()
						+ "' not supported by The EIP component.";
				getLogger().severe(error);
				exchange.setError(new NoSuchMethodException(error));
			} catch (MessagingException e) {
				getLogger().log(Level.SEVERE, e.getMessage(), e);
				exchange.setError(e);
			}
		}
	}

	/**
	 * Return the Consumes elements of the service unit that contains the
	 * provided endpoint
	 *
	 * @param endpoint
	 * @return a list of Consumes elements related to the provided endpoint
	 */
	public List<Consumes> getSUConsumes(ServiceEndpoint endpoint) {
		ServiceUnitManager sum = (ServiceUnitManager) getComponent()
				.getServiceUnitManager();
		ServiceUnitDataHandler sudh = sum.getSUDataHandlerForProvides(sum.getProvidesFromEndpoint(endpoint));
		return sudh.getDescriptor().getServices().getConsumes();
	}

	/**
	 * Process a response from a previous async send performed by the EIP
	 * component DONE and ERROR response are catch previously by the framework,
	 * so this reponse is a Fault or an OUT message. Do nothing, the framework
	 * will send back a DONE status
	 */
	protected void processAsyncResponse(Exchange exchange) {
		getLogger().info(
				"receive a asynchronous OUT or Fault response, do nothing.");
		getLogger()
				.info(
						"response received from ...\n - itf="
								+ (exchange.getEndpoint().getInterfaces().length > 0 ? exchange
										.getEndpoint().getInterfaces()[0]
										: "") + "\n - srv="
								+ exchange.getEndpoint().getServiceName()
								+ "\n - ep ="
								+ exchange.getEndpoint().getEndpointName());
	}
}
